package core.entities.timemanager;

import core.api.models.timemanager.taskSchedule.scheduleInfos.ScheduleInfo;

import javax.persistence.*;
import java.time.Duration;
import java.time.LocalDateTime;
import java.util.Objects;

@Entity
@Inheritance(strategy = InheritanceType.SINGLE_TABLE)
@DiscriminatorColumn(name="scheduleType",
        discriminatorType = DiscriminatorType.INTEGER)
public abstract class AbstractSchedule {

    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    protected long scheduleID;

    @ManyToOne
    @JoinColumn(referencedColumnName = "taskID", name = "task")
    protected Task task;

    protected LocalDateTime startTime;
    protected LocalDateTime stopTime;

    public AbstractSchedule() {
    }

    public AbstractSchedule(Task task, LocalDateTime startTime, LocalDateTime stopTime) {
        this.task = task;
        this.startTime = startTime;
        this.stopTime = stopTime;
    }

    public AbstractSchedule(Task task, LocalDateTime startTime) {
        this.task = task;
        this.startTime = startTime;
    }

    public AbstractSchedule(Task task) {
        this.task = task;
    }

    public long getScheduleID() {
        return scheduleID;
    }

    public void setScheduleID(long scheduleID) {
        this.scheduleID = scheduleID;
    }

    public Task getTask() {
        return task;
    }

    public void setTask(Task task) {
        this.task = task;
    }

    public LocalDateTime getStartTime() {
        return startTime;
    }

    public void setStartTime(LocalDateTime startTime) {
        this.startTime = startTime;
    }

    public LocalDateTime getStopTime() {
        return stopTime;
    }

    public void setStopTime(LocalDateTime stopTime) {
        this.stopTime = stopTime;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        AbstractSchedule that = (AbstractSchedule) o;
        return scheduleID == that.scheduleID && Objects.equals(task, that.task) && Objects.equals(startTime, that.startTime) && Objects.equals(stopTime, that.stopTime);
    }

    @Override
    public int hashCode() {
        return Objects.hash(scheduleID, task, startTime, stopTime);
    }

    public abstract ScheduleInfo toScheduleInfo();

    public boolean isActive() {
        return startTime != null && stopTime == null;
    }

    public boolean isCompleted() {
        return startTime != null && stopTime != null;
    }

    public int getActiveTime() {
        if(startTime == null) {
            return 0;
        } else if(stopTime == null) {
            return (int) Duration.between(startTime, LocalDateTime.now()).toMinutes();
        } else {
            return (int) Duration.between(startTime, stopTime).toMinutes();
        }
    }

    public boolean isStartable() {
        return startTime == null;
    }

    public abstract boolean isMissed(LocalDateTime timeReference);

    public int calcActiveMinutes() {
        if(startTime == null) {
            return 0;
        } else if(stopTime == null) {
            Duration duration = Duration.between(startTime, LocalDateTime.now());
            return (int) duration.toMinutes();
        } else {
            Duration duration = Duration.between(startTime, stopTime);
            return (int) duration.toMinutes();
        }
    }

    public abstract AbstractSchedule cloneSchedule();

    public abstract void shiftSchedule(long numberDays);
}
